from typing import Optional, Self

from .statistics import Statistics


class PlayerStatistics:
    _player_id: str
    _statistics: Optional[Statistics] = None

    def __init__(self, player_id: str, statistics: Statistics) -> Self:
        self._player_id = player_id
        self._statistics = statistics

    @property
    def player_id(self) -> str:
        return self._player_id

    @property
    def statistics(self) -> Statistics:
        return self._statistics

    def to_dict(self) -> dict:
        return {"player_id": self._player_id, "statistics": self._statistics.to_dict()}



from typing import Dict, Self, Tuple


class CompositeGameStatistics:
    def __init__(
        self, game_id: str = None, title: str = None, statistics: str = None, player_id: str = None
    ) -> Self:
        self.game_id = game_id
        self.title = title
        self.player_id = player_id
        self.statistics = statistics

    @classmethod
    def from_tuple(cls, composite_statistics_tuple: Tuple) -> None:
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), composite_statistics_tuple)})

    def to_dict(self) -> Dict:
        return {
            "game_id": self.game_id,
            "title": self.title,
            "player_id": self.player_id,
            "statistics": self.statistics,
        }



from datetime import datetime
from typing import List, Optional

from enums.position import Position
from pydantic import BaseModel

from .statistics import Statistics


class PlayerWithStatistics(BaseModel):
    player_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    position: Optional[Position] = None
    number: Optional[int] = 0
    statistics: Optional[Statistics] = None
    img_url: Optional[str] = "static/blank.jpg"


class GameTeamResult(BaseModel):
    team_id: Optional[str] = None
    name: Optional[str] = "Team"
    img_url: Optional[str] = "static/blank.jpg"
    roster: Optional[List[PlayerWithStatistics]] = []


class GameResult(BaseModel):
    game_id: Optional[str] = None
    title: Optional[str] = "Away @ Home"
    date: Optional[datetime] = None
    score: Optional[str] = "0-0"
    location: Optional[str] = None
    home: Optional[GameTeamResult] = GameTeamResult()
    away: Optional[GameTeamResult] = GameTeamResult()



from typing import Optional

from pydantic import BaseModel


class CoachesRequestFilters(BaseModel):
    coach_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    role: Optional[str] = None
    limit: Optional[int] = None
    offset: Optional[int] = None
    order: Optional[str] = None
    order_by: Optional[str] = None



from typing import List, Optional

from pydantic import BaseModel

from .composite_game_statistics import CompositeGameStatistics
from .composite_season_statistics import CompositeSeasonStatistics


class CompositeStatistics(BaseModel):
    games: Optional[List[CompositeGameStatistics]] = []
    season: Optional[List[CompositeSeasonStatistics]] = []



from datetime import datetime
from typing import Dict, Self, Tuple


class GameStatistics:
    def __init__(
        self,
        player_id: str = None,
        game_id: str = None,
        statistics: str = None,
        created: datetime = None,
        modified: datetime = None,
    ) -> Self:
        self.player_id = player_id
        self.game_id = game_id
        self.statistics = statistics
        self.created = created
        self.modified = modified

    @classmethod
    def from_tuple(cls, game_statistics_tuple: Tuple) -> None:
        """Create game statistics from a tuple

        Args:
            player_tuple (Tuple): Tuple containing ordered game statistic data
        """
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), game_statistics_tuple)})

    def to_dict(self) -> Dict:
        return {
            "player_id": f"{self.player_id}",
            "game_id": f"{self.game_id}",
            "statistics": f"{self.statistics}",
            "created": f"{self.created}",
            "modified": f"{self.modified}",
        }



from typing import List


class CoachValidationError(Exception):
    def __init__(self, message: str = None, invalid_fields: List[str] = None) -> None:
        self.__message = message
        self.invalid_fields = invalid_fields
        super().__init__(self.__message)


class CoachDoesNotExist(Exception):
    def __init__(self, message: str = None, invalid_coach_id: str = None) -> None:
        self.__message = message
        self.invalid_coach_id = invalid_coach_id
        super().__init__(self.__message)


class CoachAlreadyExists(Exception):
    def __init__(self, message: str = None, existing_coach_id: str = None) -> None:
        self.__message = message
        self.existing_coach_id = existing_coach_id
        super().__init__(self.__message)



from typing import List, Optional

from pydantic import BaseModel

from .statistics import Statistics


class PlayerSeasonStatistics(BaseModel):
    player_id: Optional[str] = None
    statistics: Optional[Statistics] = None


class CompositeSeasonStatistics(BaseModel):
    year: Optional[int] = None
    team_id: Optional[str] = None
    team_name: Optional[str] = None
    players: Optional[List[PlayerSeasonStatistics]] = []



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class Team(BaseModel):
    team_id: Optional[str] = None
    name: Optional[str] = None
    location: Optional[str] = None
    img_url: Optional[str] = None
    created: Optional[datetime] = None
    modified: Optional[datetime] = None



from enum import Enum


class StatisticAcronym(Enum):
    HSH = "High shots"
    MSH = "Mid shots"
    LSH = "Low shots"
    HG = "High goals"
    MG = "Mid goals"
    LG = "Low goals"
    A = "Assists"
    GB = "Ground balls"
    T = "Turnovers"
    CT = "Caused turnovers"
    P = "Penalty minutes"
    K = "Knockdowns"
    MS = "Missed slides"
    HGA = "High goals allowed"
    MGA = "Mid goals allowed"
    LGA = "Low goals allowed"
    HGS = "High goal saves"
    MGS = "Mid goal saves"
    LGS = "Low goal saves"
    FOW = "Face off wins"
    FOL = "Face off losses"



from datetime import datetime
from typing import Optional

from pydantic import BaseModel

from .statistics import Statistics


class SeasonStatistics(BaseModel):
    player_id: Optional[str] = None
    team_id: Optional[str] = None
    year: Optional[int] = None
    statistics: Optional[Statistics] = None
    created: Optional[datetime] = datetime.now()
    modified: Optional[datetime] = datetime.now()



from datetime import datetime
from typing import Self, Tuple


class Coach:
    def __init__(
        self,
        coach_id: str = None,
        first_name: str = None,
        last_name: str = None,
        role: str = None,
        since: int = None,
        email: str = None,
        phone_number: str = None,
        imgurl: str = None,
        created: datetime = None,
        modified: datetime = None,
    ) -> Self:
        self.coach_id = coach_id
        self.first_name = first_name
        self.last_name = last_name
        self.role = role
        self.since = since
        self.email = email
        self.phone_number = phone_number
        self.imgurl = imgurl
        self.created = created
        self.modified = modified

    @classmethod
    def from_tuple(cls, coach_tuple: Tuple) -> None:
        """Create coach from a tuple

        Args:
            player_tuple (Tuple): Tuple containing ordered coach data
        """
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), coach_tuple)})

    def __str__(self) -> str:
        return (
            f"{self.last_name}, {self.first_name}"
            + f"| phone: {self.phone_number}, email: {self.email}"
        )

    def to_dict(self) -> str:
        return {
            "coach_id": f"{self.coach_id}",
            "first_name": f"{self.first_name}",
            "last_name": f"{self.last_name}",
            "role": f"{self.role}",
            "since": self.since,
            "email": f"{self.email}",
            "phone_number": f"{self.phone_number}",
            "imgurl": f"{self.imgurl}",
            "created": f"{self.created}",
            "modified": f"{self.modified}",
        }



"""
Logging helper class
"""

import logging as log
import os
import sys

LOG_DIR = str(os.getcwd()) + "/logs/"


def log_setup(logger_name: str, log_file: str, mode: str = "a"):
    """
    Configure a new logger and return the new instance to the user

    Args:
        logger_name (str): User defined name for the Logger obj instance
        log_file (str): Name for the log_file
        mode (str, optional): Log mode. Recommended: 'w'rite or 'a'ppend. Defaults to 'w'.

    Returns:
        log.Logger: New log file instance the user can write to
    """

    #   Initialize handlers
    new_log = log.getLogger(logger_name)
    formatter = log.Formatter("[%(levelname)s]\t[%(asctime)s] %(message)s")
    file_handler = log.FileHandler(log_file, mode=mode)
    file_handler.setFormatter(formatter)
    stream_handler = log.StreamHandler(sys.stdout)

    #   Create logging object with handlers
    new_log.setLevel(log.DEBUG)
    new_log.addHandler(file_handler)
    new_log.addHandler(stream_handler)
    return new_log


class Logger:
    """
    Logger object that allows a user to quickly define a new instance and log results to the file
    """

    def __init__(self, key: str = "none"):
        """Constructor for log object. Takes a name for the file and handles the rest of the
        setup internally.

        Args:
            key (str, optional): Assignment/Name of the logger. Defaults to "none".
        """
        self.log_obj = log_setup(f"{key}", LOG_DIR + f"{key}.log")

    def __call__(self, log_str: str, mode: str = "i"):
        """
        Call the object to have a message immediately logged to the debug output

        Args:
            log_str (str): Message to add to the logfile
            mode (str, optional): Logging mode for the file. Defaults to 'i' for Info
        """
        if mode == "i":
            self.log_obj.info(log_str)
        elif mode == "d":
            self.log_obj.debug(log_str)
        elif mode == "e":
            self.log_obj.error(log_str)
        elif mode == "w":
            self.log_obj.warning(log_str)
        else:
            self.log_obj.info(log_str)

    def log(self, log_str: str):
        """
        Log a debug string to the specified log file

        Args:
            log_str (str): String to add to logfile
        """
        self.log_obj.debug(log_str)

    def info(self, log_str: str):
        """
        Log an info string to the specified log file

        Args:
            log_str (str): String to add to logfile
        """
        self.log_obj.info(log_str)

    def error(self, log_str: str):
        """Log an error string to the log file

        Args:
            log_str (str): String to add to the logfile
        """
        self.log_obj.error(log_str)

    def debug(self, log_str: str):
        """Log a debug string to the log file

        Args:
            log_str (str): String to add to the logfile
        """
        self.log_obj.debug(log_str)

    def warning(self, log_str: str):
        """Log a warning string to the log file

        Args:
            log_str (str): String to add to the logfile
        """
        self.log_obj.warning(log_str)



from typing import List, Self

from models.composite_season_statistics import CompositeSeasonStatistics
from models.player_statistics import PlayerStatistics


class CompositeSeasonStatistics:
    _year: int


class CompositeSeasonStatisticsFull(CompositeSeasonStatistics):
    _team_id: str
    _team_name: str
    _players: List[PlayerStatistics]

    def __init__(
        self, year: int, team_id: str, team_name: str, players: List[PlayerStatistics]
    ) -> Self:
        super().__init__()
        self._year = year
        self._team_id = team_id
        self._team_name = team_name
        self._players = players

    @property
    def year(self) -> int:
        return self._year

    @property
    def team_id(self) -> str:
        return self._team_id

    @property
    def team_name(self) -> str:
        return self._team_name

    @property
    def players(self) -> List[PlayerStatistics]:
        return self._players

    def to_dict(self) -> dict:
        return {
            "year": self._year,
            "team_id": self._team_id,
            "team_name": self._team_name,
            "players": [player_stats.to_dict() for player_stats in self.players],
        }


class CompositeSeasonByYear(CompositeSeasonStatistics):
    _data: List[CompositeSeasonStatisticsFull]

    def __init__(self, year: int, data: List[CompositeSeasonStatisticsFull]) -> None:
        self._year = year
        self._data = data

    @property
    def year(self) -> int:
        return self._year

    @property
    def data(self) -> List[CompositeSeasonStatisticsFull]:
        return self._data

    def to_dict(self) -> dict:
        return {
            "year": self._year,
            "data": [data.to_dict() for data in self._data],
        }



from typing import List


class StatisticsValidationError(Exception):
    def __init__(self, message=None, invalid_fields: List = None):
        self.__message = message
        self.invalid_fields = invalid_fields
        super().__init__(self.__message)


class StatisticsDoNoExist(Exception):
    def __init__(self, message=None):
        self.__message = message
        super().__init__(self.__message)


class StatisticsAlreadyExist(Exception):
    def __init__(self, message=None) -> None:
        self.__message = message
        super().__init__(self.__message)


class GameStatisticsValidationError(Exception):
    def __init__(self, message=None, invalid_fields: List = None):
        self.__message = message
        self.invalid_fields = invalid_fields
        super().__init__(self.__message)


class GameStatisticsDoNoExist(Exception):
    def __init__(self, message=None):
        self.__message = message
        super().__init__(self.__message)


class GameStatisticsAlreadyExist(Exception):
    def __init__(self, message=None) -> None:
        self.__message = message
        super().__init__(self.__message)



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class TeamCoach(BaseModel):
    team_id: Optional[str] = None
    coach_id: Optional[str] = None
    year: Optional[int] = None
    role: Optional[str] = None
    since: Optional[int] = None
    created: Optional[datetime] = None
    modified: Optional[datetime] = None



from typing import Dict, Optional, Self
from urllib.parse import urlencode

from client.client_url import ClientUrl

from .ordering import OrderingRules
from .pagination import Pagination


class PlayerDataServiceRequest:
    _url: ClientUrl
    _path_parameters: Optional[Dict]
    _query_parameters: Optional[Dict]
    _request_body: Optional[Dict]
    _pagination: Optional[Pagination]
    _ordering: Optional[OrderingRules]

    def __init__(
        self,
        url: ClientUrl,
        path_parameters: str = None,
        query_parameters: Dict = None,
        request_body: Dict = None,
        pagination: Pagination = None,
        ordering: OrderingRules = None,
    ) -> Self:
        self._url = url
        self._path_parameters = path_parameters
        self._query_parameters = query_parameters if query_parameters is not None else dict()
        self._request_body = request_body if request_body is not None else dict()
        self._pagination = pagination if pagination is not None else Pagination()
        self._ordering = ordering if ordering is not None else OrderingRules()

    def construct_query_parameters(self) -> Dict:
        return self.query_parameters | self.pagination.to_dict() | self.ordering.to_dict()

    def query_string(self) -> str:
        return "?" + urlencode(self.construct_query_parameters())

    @property
    def uri(self) -> str:
        uri = self._url.url

        if self.path_parameters is not None and len(self.path_parameters) > 0:
            uri += f"/{self.path_parameters}"

        if self.query_parameters is not None and len(self.query_parameters) > 0:
            uri += self.query_string()

        return uri

    @property
    def path_parameters(self) -> Dict:
        return self._path_parameters

    @path_parameters.setter
    def path_parameters(self, new_path_parameters: Dict) -> Dict:
        self.path_parameters = new_path_parameters

    @property
    def query_parameters(self) -> Dict:
        return self._query_parameters

    @query_parameters.setter
    def query_parameters(self, new_query_params: Dict) -> Dict:
        self._query_parameters = new_query_params

    @property
    def request_body(self) -> Dict:
        return self._request_body

    @request_body.setter
    def request_body(self, new_request_body: Dict) -> None:
        self._request_body = new_request_body

    @property
    def limit(self) -> int:
        return self._pagination.limit

    @limit.setter
    def limit(self, new_limit: int) -> None:
        self._pagination.limit = new_limit

    @property
    def offset(self) -> int:
        return self._pagination.offset

    @offset.setter
    def offset(self, new_offset: int) -> None:
        self._pagination.offset = new_offset

    @property
    def order(self) -> str:
        return self._ordering.order

    @order.setter
    def order(self, new_order: str) -> None:
        self._ordering.order = new_order

    @property
    def order_by(self) -> str:
        return self._ordering.order_by

    @order_by.setter
    def order_by(self, new_order_by: str) -> None:
        self._ordering.order_by = new_order_by

    @property
    def pagination(self) -> Pagination:
        return self._pagination

    @property
    def ordering(self) -> OrderingRules:
        return self._ordering

    @property
    def url(self) -> ClientUrl:
        return self._url

    @url.setter
    def url(self, new_url: ClientUrl) -> None:
        self._url = new_url

    @property
    def method(self) -> str:
        return self._url.method

    @method.setter
    def method(self, new_method: str) -> None:
        self._url.method = new_method



from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class Game(BaseModel):
    game_id: Optional[str] = None
    title: Optional[str] = None
    date: Optional[datetime] = None
    score: Optional[str] = None
    location: Optional[str] = None
    created: Optional[datetime] = datetime.now()
    modified: Optional[datetime] = datetime.now()

    @property
    def date(self) -> str:
        return datetime.fromisoformat(self.date).strftime("%A \t %m/%d/%Y \t - \t %I:%M %p")



from typing import Dict, Self, Tuple


class CompositeSeasonStatistics:
    def __init__(
        self,
        team_id: str = None,
        name: str = None,
        player_id: str = None,
        year: int = None,
        statistics: str = None,
    ) -> Self:
        self.team_id = team_id
        self.name = name
        self.player_id = player_id
        self.year = year
        self.statistics = statistics

    @classmethod
    def from_tuple(cls, composite_statistics_tuple: Tuple) -> None:
        return cls(**{k: v for k, v in zip(cls().to_dict().keys(), composite_statistics_tuple)})

    def to_dict(self) -> Dict:
        return {
            "team_id": self.team_id,
            "name": self.name,
            "player_id": self.player_id,
            "year": self.year,
            "statistics": self.statistics,
        }



from typing import List


class PlayerRequestValidationError(Exception):
    def __init__(self, message=None, invalid_fields: List[str] = None):
        self.__message = message
        self.invalid_fields = invalid_fields
        super().__init__(self.__message)


class PlayerDoesNotExist(Exception):
    def __init__(self, message=None, invalid_player_id: str = None) -> None:
        self.__message = message
        self.invalid_player_id = invalid_player_id
        super().__init__(self.__message)


class PlayerAlreadyExists(Exception):
    def __init__(self, message=None, existing_player_id: str = None) -> None:
        self.__message = message
        self.existing_player_id = existing_player_id
        super().__init__(self.__message)



from datetime import datetime
from typing import Optional

from enums.position import Position
from pydantic import BaseModel


class TeamPlayer(BaseModel):
    team_id: Optional[str] = None
    player_id: Optional[str] = None
    year: Optional[int] = None
    number: Optional[int] = None
    position: Optional[Position] = None
    created: Optional[datetime] = None
    modified: Optional[datetime] = None
